searchData={"items":[{"type":"behaviour","title":"PhoenixPages","doc":"Blogs, docs, and static pages in Phoenix. Check out the [README](readme.html) to get started.","ref":"PhoenixPages.html"},{"type":"behaviour","title":"Options - PhoenixPages","doc":"* `:otp_app` - The name of the OTP application to use as the base directory when looking for\n  page files. This value is required.\n\n  * `:render_options` - Allows the renderers to be configured. See `pages/4` for the options.","ref":"PhoenixPages.html#module-options"},{"type":"callback","title":"PhoenixPages.get_pages/1","doc":"Gets a list of pages for a given ID. See the `:id` option in `pages/4`.\n\nThis can be used to get a list of pages from within a controller for index pages or further\ncustomization such as finding related pages, filtering by a value, etc. Returns `{:ok, pages}`\nif successful, or `:error` if no pages were defined with the ID.","ref":"PhoenixPages.html#c:get_pages/1"},{"type":"callback","title":"PhoenixPages.get_pages!/1","doc":"Same as `get_pages/1`, but will raise `PhoenixPages.NotFoundError` if no pages were defined with\nthe ID.","ref":"PhoenixPages.html#c:get_pages!/1"},{"type":"type","title":"PhoenixPages.page/0","doc":"","ref":"PhoenixPages.html#t:page/0"},{"type":"macro","title":"PhoenixPages.pages/4","doc":"Finds all the pages matching a pattern and generates a GET route for each one.","ref":"PhoenixPages.html#pages/4"},{"type":"macro","title":"Options - PhoenixPages.pages/4","doc":"* `:id` - The ID for this collection of pages so they can later be accessed with\n  [`get_pages/1`](PhoenixPages.html#c:get_pages/1) and [`get_pages!/1`](PhoenixPages.html#c:get_pages!/1).\n  This is only required if using those functions.\n\n  * `:from` - The [wildcard pattern](https://hexdocs.pm/elixir/1.13/Path.html#wildcard/2) used\n  to look for pages on the filesystem. Make sure the base directory is included in your release\n  (`priv` is included by default). Defaults to `priv/pages/**/*.md`.\n\n  * `:sort` - The order of the pages returned when using `get_pages/1` and `get_pages!/1`.\n  Defined as a tuple with the first element being an atom for the sort value (which can be any\n  value from the attributes), and the second element being either `:asc` or `:desc`.\n\n  * `:attrs` - A list of attributes used when parsing the markdown frontmatter for each page.\n  Atoms will be required for each page, key values will be optional with a default value.\n  Defaults to `[]`.\n\n  * `:render_options` - Allows the renderers to be configured. This can also be set globally in\n  the [module options](PhoenixPages.html#module-options).\n\n    * `:markdown` - Allows the Earmark renderer to be configured.\n\n      * `:hard_breaks` - Whether to convert hard line breaks to ` ` tags. Defaults to `false`.\n      * `:wiki_links` - Whether to enable wiki-style links such as `[[page]]`. Defaults to `true`.\n      * `:pure_links` - Whether to convert raw URLs to ` ` tags. Defaults to `true`.\n      * `:sub_sup` - Whether to convert `~x~` and `^x^` to ` ` and ` ` tags. Defaults to `true`.\n      * `:footnotes` - Whether to enable footnotes. Defaults to `true`.\n      * `:smartypants` - Whether to enable [smartypants](https://daringfireball.net/projects/smartypants/). Defaults to `true`.\n      * `:compact_output` - Whether to avoid indentation and minimize whitespace in output. Defaults to `false`.\n      * `:escape_html` - Whether to allow raw HTML in markdown. Defaults to `false`.\n      * `:syntax_highlighting` - Whether to enable Makeup syntax highlighting. Defaults to `true`.\n      * `:code_class_prefix` - A class prefix to add to the language class of code blocks.\n\n  * All other options are passed to [`Phoenix.Router.match/5`](https://hexdocs.pm/phoenix/Phoenix.Router.html#match/5-options)","ref":"PhoenixPages.html#pages/4-options"},{"type":"module","title":"PhoenixPages.Page","doc":"","ref":"PhoenixPages.Page.html"},{"type":"type","title":"PhoenixPages.Page.t/0","doc":"","ref":"PhoenixPages.Page.html#t:t/0"},{"type":"exception","title":"PhoenixPages.NotFoundError","doc":"Exception raised when `get_pages/1` or `get_pages!/1` is called with an ID that was not found.","ref":"PhoenixPages.NotFoundError.html"},{"type":"exception","title":"PhoenixPages.ParseError","doc":"Exception raised when a page's frontmatter cannot be parsed.","ref":"PhoenixPages.ParseError.html"},{"type":"extras","title":"Phoenix Pages","doc":"# Phoenix Pages\n\nAdd blogs, documentation, and other static pages to Phoenix apps. This library integrates seamlessly into your router and comes with built-in support for rendering markdown with frontmatter, syntax highlighting, compile-time caching, and more.\n\n- [Installation](#installation)\n- [Getting Started](#getting-started)\n- [Frontmatter](#frontmatter)\n- [Syntax Highlighting](#syntax-highlighting)\n- [Index Pages](#index-pages)\n- [Defining Paths](#defining-paths)\n- [Extended Markdown](#extended-markdown)\n- [Local Development](#local-development)","ref":"readme.html"},{"type":"extras","title":"Installation - Phoenix Pages","doc":"```elixir\ndef deps do\n  [\n    {:phoenix_pages, \"~> 0.1\"}\n  ]\nend\n```\n\nThe recommended way to install into your Phoenix application is to add this to your `router` function in `lib/myapp_web.ex`, replacing `myapp` with the name of your application:\n\n```elixir\ndef router do\n  quote do\n    use Phoenix.Router, helpers: false\n    use PhoenixPages, otp_app: :myapp\n\n    # ...\n  end\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Getting Started - Phoenix Pages","doc":"Now you can add a new route using the [`pages/4`](https://hexdocs.pm/phoenix_pages/PhoenixPages.html#pages/4) macro:\n\n```elixir\nscope \"/\", MyAppWeb do\n  pipe_through :browser\n\n  get \"/\", PageController, :home\n  pages \"/:page\", PageController, :show, from: \"priv/pages/**/*.md\"\nend\n```\n\nThis will read all the markdown files from `priv/pages` and create a new GET route for each one. The `:page` segment will be replaced with the path and filename (without the extension) relative to the base directory (see [Defining Paths](#defining-paths)).\n\nYou'll also need to add the `:show` handler to `lib/myapp_web/controllers/page_controller.ex`:\n\n```elixir\ndefmodule MyAppWeb.PageController do\n  use MyAppWeb, :controller\n\n  # ...\n\n  def show(conn, _params) do\n    render(conn, \"show.html\")\n  end\nend\n```\n\nLastly, add a template at `lib/myapp_web/controllers/page_html/show.html.heex`. The page's rendered markdown will be available in the `inner_content` assign:\n\n```heex\n \n  <%= @inner_content %>\n \n```\n\nThat's it! Now try creating a file at `priv/pages/hello.md` and visiting `/hello`.","ref":"readme.html#getting-started"},{"type":"extras","title":"Formatting - Phoenix Pages","doc":"To prevent `mix format` from adding parenthesis to the `pages` macro similar to the other Phoenix Router macros, add `:phoenix_pages` to `.formatter.exs`:\n\n```elixir\n[\n  import_deps: [:ecto, :ecto_sql, :phoenix, :phoenix_pages]\n]\n```","ref":"readme.html#formatting"},{"type":"extras","title":"Frontmatter - Phoenix Pages","doc":"Frontmatter allows page-specific variables to be included at the top of a markdown file using the YAML format. If you're setting frontmatter variables (which is optional), they must be the first thing in the file and must be set between triple-dashed lines:\n\n```markdown\n---\ntitle: Hello World\n---\nLorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut\nlabore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris\nnisi ut aliquip ex ea commodo consequat.\n```\n\nTo specify which frontmatter values are expected in each page, set the `attrs` option:\n\n```elixir\npages \"/:page\", PageController, :show,\n  from: \"priv/pages/**/*.md\",\n  attrs: [:title, author: nil]\n```\n\nAtom values will be considered required, and a compilation error will be thrown if missing from any of the pages. Key-values must come last in the list, and will be considered optional by defining a default value. Any frontmatter values not defined in the attributes list will be silently discarded.\n\nValid attribute values will be available in the assigns:\n\n```heex\n \n   <%= @title %> \n   <%= @author %> \n\n  <%= @inner_content %>\n \n```","ref":"readme.html#frontmatter"},{"type":"extras","title":"Syntax Highlighting - Phoenix Pages","doc":"Phoenix Pages uses the [Makeup](https://github.com/elixir-makeup/makeup) project for syntax highlighting. To enable, add a lexer for your specific language(s) to the project dependencies. Phoenix Pages will pick up the new dependency and start highlighting your code blocks without any further configuration. No lexers are included by default.\n\n \n    Lexers  \n\n   \n      C  -  `{:makeup_c, \"~> 0.0\"}`  \n      Diff  -  `{:makeup_diff, \"~> 0.0\"}`  \n      Elixir  -  `{:makeup_elixir, \"~> 0.0\"}`  \n      Erlang  -  `{:makeup_erlang, \"~> 0.0\"}`  \n      GraphQL  -  `{:makeup_graphql, \"~> 0.0\"}`  \n      (H)EEx  -  `{:makeup_eex, \"~> 0.0\"}`  \n      HTML  -  `{:makeup_html, \"~> 0.0\"}`  \n      Javascript  -  `{:makeup_js, \"~> 0.0\"}`  \n      JSON  -  `{:makeup_json, \"~> 0.0\"}`  \n      Rust  -  `{:makeup_rust, \"~> 0.0\"}`  \n      SQL  -  `{:makeup_sql, \"~> 0.0\"}`  \n   \n \n\nIf your language of choice isn't supported, consider [writing a new Makeup lexer](https://github.com/elixir-makeup/makeup/blob/master/CONTRIBUTING.md#writing-a-new-lexer) to contribute to the community. Otherwise, you can use a JS-based syntax highlighter such as [highlight.js](https://highlightjs.org) by setting `code_class_prefix: \"language-\"` and `syntax_highlighting: false` in [`render_options`](https://hexdocs.pm/phoenix_pages/PhoenixPages.html#pages/4-options).\n\nNext, import a theme listed below into your CSS bundle. The specifics of doing this highly depend on your CSS configuration, but a few examples are included below. In most cases, you will need to import `phoenix_pages/css/monokai.css` (or whatever theme you choose) into your bundle and ensure `deps` is included as a vendor directory.\n\n \n    Themes  \n\n   \n      abap  \n      algol  \n      algol_nu  \n      arduino  \n      autumn  \n      black_white  \n      borland  \n      colorful  \n      default  \n      emacs  \n      friendly  \n      fruity  \n      igor  \n      lovelace  \n      manni  \n      monokai  \n      murphy  \n      native  \n      paraiso_dark  \n      paraiso_light  \n      pastie  \n      perldoc  \n      rainbow_dash  \n      rrt  \n      samba  \n      tango  \n      trac  \n      vim  \n      visual_studio  \n      xcode","ref":"readme.html#syntax-highlighting"},{"type":"extras","title":"ESBuild Example - Phoenix Pages","doc":"Using the [ESBuild installer](https://github.com/phoenixframework/esbuild), add the `env` option to `config/config.exs`:\n\n```elixir\nconfig :esbuild,\n  version: \"0.17.18\",\n  default: [\n    cd: Path.expand(\"../assets\", __DIR__),\n    env: %{\"NODE_PATH\" => Path.expand(\"../deps\", __DIR__)},\n    args: ~w(--bundle --outdir=../priv/static/assets js/app.js)\n  ]\n```\n\nThen in `app.js`:\n\n```javascript\nimport \"phoenix_pages/css/monokai.css\";\n```","ref":"readme.html#esbuild-example"},{"type":"extras","title":"Sass Example - Phoenix Pages","doc":"Using the [Sass installer](https://github.com/CargoSense/dart_sass), add the `--load-path` flag to `config/config.exs`:\n\n```elixir\nconfig :dart_sass,\n  version: \"1.62.0\",\n  default: [\n    cd: Path.expand(\"../assets\", __DIR__),\n    args: ~w(--load-path=../deps css/app.scss ../priv/static/assets/app.css)\n  ]\n```\n\nThen in `app.scss`:\n\n```scss\n@import \"phoenix_pages/css/monokai\";\n```","ref":"readme.html#sass-example"},{"type":"extras","title":"Tailwind Example - Phoenix Pages","doc":"Install the `postcss-import` plugin as described [here](https://tailwindcss.com/docs/using-with-preprocessors#build-time-imports) and add the following to `assets/postcss.config.js`:\n\n```javascript\nmodule.exports = {\n  plugins: {\n    \"postcss-import\": {}\n  }\n}\n```\n\nThen in `app.css`:\n\n```css\n@import \"../../deps/phoenix_pages/css/monokai\";\n```","ref":"readme.html#tailwind-example"},{"type":"extras","title":"Index Pages - Phoenix Pages","doc":"To create an index page with links to all the other pages, create a normal GET route and use the [`id`](https://hexdocs.pm/phoenix_pages/PhoenixPages.html#pages/4-options) option alongside [`get_pages/1`](https://hexdocs.pm/phoenix_pages/PhoenixPages.html#c:get_pages/1) and [`get_pages!/1`](https://hexdocs.pm/phoenix_pages/PhoenixPages.html#c:get_pages!/1) in your router:\n\n```elixir\nget \"/blog\", BlogController, :index\n\npages \"/blog/:page\", BlogController, :show,\n  id: :blog,\n  from: \"priv/blog/**/*.md\",\n  attrs: [:title, :author, :date]\n```\n\n```elixir\ndefmodule MyAppWeb.BlogController do\n  use MyAppWeb, :controller\n\n  def index(conn, _params) do\n    pages = MyAppWeb.Router.get_pages!(:blog)\n\n    conn\n    |> assign(:pages, pages)\n    |> render(\"index.html\")\n  end\n\n  def show(conn, _params) do\n    render(conn, \"show.html\")\n  end\nend\n```\n\n```heex\n<.link :for={page <- @pages} navigate={page.path}>\n  <%= page.assigns.title %>\n</.link>\n```\n\nAll the page files are read and cached during compilation, so the `get_pages` functions will not actually read anything from the filesystem—making them very performant.","ref":"readme.html#index-pages"},{"type":"extras","title":"Sorting - Phoenix Pages","doc":"The pages returned from the `get_pages` functions will be sorted by filename. If you want to specify a different order during compilation rather than in the controller on every page load, use the [`sort`](https://hexdocs.pm/phoenix_pages/PhoenixPages.html#pages/4-options) option:\n\n```elixir\npages \"/blog/:page\", BlogController, :show,\n  id: :blog,\n  from: \"priv/blog/**/*.md\",\n  attrs: [:title, :author, :date],\n  sort: {:date, :desc}\n```\n\nAny attribute value from the frontmatter can be defined as the sort value.","ref":"readme.html#sorting"},{"type":"extras","title":"Defining Paths - Phoenix Pages","doc":"When defining the pages path, the `:page` segment will be replaced for each generated page **during compilation** with the values derived from `**` and `*`. This is different than segments in regular routes, which are parsed **during runtime** into the `params` attribute of the controller function.\n\nFor example, let's say you have the following file structure:\n\n```\n┌── priv/\n│  ┌── pages/\n│  │  ┌── foo.md\n│  │  ├── bar/\n│  │  │  ┌── baz.md\n```\n\nDefining `pages \"/:page\", from: \"priv/pages/**/*.md\"` in your router will create two routes: `get \"/foo\"` and `get \"/bar/baz\"`. You can even put the `:page` segment somewhere else in the path, such as `/blog/:page`, and it will work as expected creating `get \"/blog/foo\"` and `get \"/blog/bar/baz\"`.","ref":"readme.html#defining-paths"},{"type":"extras","title":"Capture Groups - Phoenix Pages","doc":"For complex scenarios, you have the option of using capture group variables instead of the `:page` segment.\n\nLet's say you have the same file structure as above, but don't want the `baz` path to be nested under `/bar`. You could define `pages \"/$2\", from: \"priv/pages/**/*.md\"`, using `$2` instead of `:page`. This will create two routes: `get \"/foo\"` and `get \"/bar\"`.\n\nCapture group variables will contain the value of the `**` and `*` chunks in order, starting at `$1`. Keep in mind that `**` will match all files and zero or more directories and subdirectories, and `*` will match any number of characters up to the end of the filename, the next dot, or the next slash.\n\nFor more info on the wildcard patterns, check out [Path.wildcard/2](https://hexdocs.pm/elixir/1.13/Path.html#wildcard/2).","ref":"readme.html#capture-groups"},{"type":"extras","title":"Extended Markdown - Phoenix Pages","doc":"In addition to the customizable [markdown options](https://hexdocs.pm/phoenix_pages/PhoenixPages.html#pages/4-options), markdown rendering also supports IAL attributes by default. Meaning you can add HTML attributes to any block-level element using the syntax `{:attr}`.\n\nFor example, to create a rendered output of ` Header `:\n\n```markdown\n# Header{:.foobar}\n```\n\nAttributes can be one of the following:\n\n- `{:#id}` to define an ID\n- `{:.className}` to define a class name\n- `{:name=value}`, `{:name=\"value\"}`, or `{:name='value'}` to define any other attribute\n\nTo define multiple attributes, separate them with spaces: `{:#id name=value}`.","ref":"readme.html#extended-markdown"},{"type":"extras","title":"Local Development - Phoenix Pages","doc":"If you add, remove, or change pages while running `mix phx.server`, they will automatically be replaced in the cache and you don't have to restart for them to take effect. To live reload when a page changes, add to the patterns list of the Endpoint config in `config/dev.exs`:\n\n```elixir\nconfig :myapp, MyAppWeb.Endpoint,\n  live_reload: [\n    patterns: [\n      # ...\n      ~r\"priv/pages/.*(md)$\",\n      # ...\n    ]\n  ]\n```","ref":"readme.html#local-development"}],"content_type":"text/markdown","proglang":"elixir","producer":{"name":"ex_doc","version":"0.37.0"}}